@@ -1,5 +1,5 @@
 /*
- * Binary node program copyright (C) 2009 H.Niwa 
+ * Binary node program copyright (C) 2009 - 2011 H.Niwa 
  */
 
 /*
@@ -34,8 +34,6 @@
 
 #include "bin_node.h"
 
-Node* AllNodes = NULL;
-
 unsigned int PrCount = 0;
 
 // The flag which prints a double quote on atom
@@ -49,10 +47,100 @@ Node* True = &Truearea;
 Atom  Nilarea("()");
 Node* Nil = &Nilarea;
 
+Atom  AllNodesBody("AllNodes");
+Node* AllNodes = &AllNodesBody;
+
+Node* FreeListNodes = NULL;
+Node* FreeAtomNodes = NULL;
+
+static inline Node* NewList(Node* nd1, Node* nd2)
+{
+	if (FreeListNodes == NULL) {
+		return new List(nd1, nd2);
+	} else {
+		Node* nd = FreeListNodes;
+		FreeListNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((List*)nd)->Set(nd1, nd2);
+		return nd;
+	}
+}
+
+Node* NewAtom(char const* s)
+{
+	if (FreeAtomNodes == NULL) {
+		return new Atom(s);
+	} else {
+		Node* nd = FreeAtomNodes;
+		FreeAtomNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Atom*)nd)->Set(s);
+		return nd;
+	}
+}
+
+Node* NewAtom(char* s)
+{
+	if (FreeAtomNodes == NULL) {
+		return new Atom(s);
+	} else {
+		Node* nd = FreeAtomNodes;
+		FreeAtomNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Atom*)nd)->Set(s);
+		return nd;
+	}
+}
+
+Node* NewAtom(long long n)
+{
+	if (FreeAtomNodes == NULL) {
+		return (Node*)(new Atom(n));
+	} else {
+		Node* nd = FreeAtomNodes;
+		FreeAtomNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Atom*)nd)->Set(n);
+		return nd;
+	}
+}
+
+Node* NewAtom(long double n)
+{
+	if (FreeAtomNodes == NULL) {
+		return (Node*)(new Atom(n));
+	} else {
+		Node* nd = FreeAtomNodes;
+		FreeAtomNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Atom*)nd)->Set(n);
+		return nd;
+	}
+}
+
+Node* NewAtom(std::complex<long double> c)
+{
+	if (FreeAtomNodes == NULL) {
+		return (Node*)(new Atom(c));
+	} else {
+		Node* nd = FreeAtomNodes;
+		FreeAtomNodes = nd->nextnode;
+		nd->nextnode = AllNodes->nextnode;
+		AllNodes->nextnode = nd;
+		((Atom*)nd)->Set(c);
+		return nd;
+	}
+}
+
 
 Node* Cons(Node* nd1, Node* nd2) 
 {
-	return new List(nd1, nd2);
+	return NewList(nd1, nd2);
 }
 
 
@@ -129,7 +217,7 @@ Node* Append(Node* nd1, Node* nd2)
 // Node functions
 Node* Node::Cons(Node* nd)
 {
-	return new List(this, nd);
+	return NewList(this, nd);
 }
 
 // Atom functions
@@ -156,6 +244,8 @@ long long Atom::toInt(long long& n)
 */
 	if (*ep != 0) {
 		return 0;
+	} else if (str->c_str() == ep) {
+		return 0;
 	}else {
 		n = i;
 		return 1;
@@ -167,6 +257,7 @@ int Atom::toFloat(long double& f)
 {
 	char* ep;
 	long double d;
+	errno = 0;
 
 	if (floatvalue_flag) {
 		f = floatvalue;
@@ -180,7 +271,9 @@ int Atom::toFloat(long double& f)
 
 	if (errno == ERANGE) {
 		return 0;
-	} if (*ep != 0) {
+	} else if (str->c_str() == ep) {
+		return 0;
+	} else if (*ep != 0) {
 		return 0;
 	} else {
 		f = d;
@@ -279,7 +372,7 @@ void Atom::printatom(FILE* fd, char* s)
 	int flg=0;
 
 	if (strlen(s) == 0) {
-		printf("\"\"");
+//		printf("\"\"");
 		return;
 	} 
 #if 1
@@ -396,6 +489,28 @@ void Atom::printatom(FILE* fd, char* s)
 
 
 // List functions
+#ifdef __CYGWIN__
+inline Node* List::Val()
+{
+	Node* nd = Nil->Cons(Nil);
+	Node* ndp = nd;
+	Node* nl = this;
+	Node* newndp = Nil;
+
+	while (nl->Cdr()->kind() == LIST) {
+		newndp = Nil->Cons(Nil);
+		((List*)ndp)->Set(nl->Car()->Val(), newndp);
+		ndp = newndp;
+		nl = nl->Cdr();
+	}
+
+	((List*)ndp)->Set(nl->Car()->Val(), nl->Cdr()->Val());
+	return nd;
+
+	return Car()->Val()->Cons(Cdr()->Val());
+}
+#endif
+
 int List::Eq(Node* nd)
 {
 	nd = nd->Val();
@@ -488,14 +603,14 @@ void PrintNode(Node* nd)
 	printf("\n");
 }
 
-void PrintNode(char* s, Node* nd)
+void PrintNode(const char* s, Node* nd)
 {
 	printf("%s", s);
 	nd->print();
 	printf("\n");
 }
 
-void PrintNode(char* s1, Node* nd, char* s2)
+void PrintNode(const char* s1, Node* nd, const char* s2)
 {
 	printf("%s", s1);
 	nd->print();
