@@ -1,5 +1,5 @@
 /*
- * built-in program copyright (C) 2009 - 2010 H.Niwa
+ * built-in program copyright (C) 2009 - 2011 H.Niwa
  */
 
 
@@ -97,6 +97,7 @@ extern int PopStack(Context* cx, Node* &goals, Node* &md, Node* &env);
 extern FILE* MksTemp(char* templ);
 
 extern int BreakFlag;
+extern int ThrowFlag;
 
 int ListLength(Node* n);
 
@@ -128,6 +129,10 @@ int Warn(Context* cx, Node* goalscar, List* module);
 
 int GetModule(Context* cx, Node* goalscar, List* module);
 int DoUnify(Context* cx, Node* goalscar, List* module);
+int Catch(Context* cx, Node* goalscar, List* module);
+int Catch_SIGINT(Context* cx, Node* goalscar, List* module);
+int EventThrow(Node* ev);
+int Throw(Context* cx, Node* goalscar, List* module);
 int TimeOut(Context* cx, Node* goalscar, List* module);
 int DoLoop(Context* cx, Node* goalscar, List* module);
 int DoOrPred(Context* cx, Node* goalscar, List* module);
@@ -455,6 +460,15 @@ int builtin(Context* cx, Node* goalscar, Node* &goalscdr,
 		} else if (s == "unify") {
 			r = DoUnify(cx, goalscar, module);
 			return 1;
+		} else if (s == "catch") {
+			r = Catch(cx, goalscar, module);
+			return 1;
+		} else if (s == "catch_sigint") {
+			r = Catch_SIGINT(cx, goalscar, module);
+			return 1;
+		} else if (s == "throw") {
+			r = Throw(cx, goalscar, module);
+			return 1;
 		} else if (s == "timeout") {
 			r = TimeOut(cx, goalscar, module);
 			return 1;
@@ -738,7 +752,7 @@ int xex(Context* cx, Node* goalscar, List* module)
 		PrintNode("x : ", goalscar);
 	}
 	
-	Context* cx2 = new Context(module);
+	Context* cx2 = new Context(module, cx->modulename);
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
@@ -811,7 +825,7 @@ int IncludeSub(Context* cx, Node* goalscar, List* module, char* libname)
 		fd = fopen(sfile.c_str(), "rb");
 	}
 
-	char* fname;
+	char* fname = NULL;
 	if (fd == NULL) {
 		fd = lib_include((char*)sfile.c_str(), fname);
 		if (fd == NULL) {
@@ -847,7 +861,9 @@ int IncludeSub(Context* cx, Node* goalscar, List* module, char* libname)
 	fclose(fd);
 	RdFp = fdsave;
 
-	unlink(fname);
+	if (fname != NULL) {
+		unlink(fname);
+	}
 		
 	incflag = 1;
 	return 1;
@@ -1678,7 +1694,20 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 					Node* nvar = g->Car()->Cdr()->Car();
 					Node* val = g->Car()->Cdr()->Cdr()->Car();
 					Node* index = g->Car()->Cdr()->Cdr()->Cdr()->Car();
-					return ObjSetArray(nobj, nvar, val, index, module);
+					int r = ObjSetArray(nobj, nvar, val, index, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <setArray ");
+							nvar->print();
+							printf(" ");
+							val->print();
+							printf(" ");
+							index->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				}
 			}
 		}
@@ -1693,14 +1722,36 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 					Node* nobj = marg;
 					Node* nvar = g->Car()->Cdr()->Car();
 					Node* val  = g->Car()->Cdr()->Cdr()->Car();
-					return ObjSetVar(nobj, nvar, val, module);
+					int r = ObjSetVar(nobj, nvar, val, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <setVar ");
+							nvar->print();
+							printf(" ");
+							val->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "delArray") {
 					Node* nobj = marg;
 					Node* nvar  = g->Car()->Cdr()->Car();
 					Node* index = g->Car()->Cdr()->Cdr()
 							->Car();
-					return ObjDelArray(nobj, nvar, index, 
+					int r = ObjDelArray(nobj, nvar, index, 
 								module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <delArray ");
+							nvar->print();
+							printf(" ");
+							index->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				}
 			}
 		}
@@ -1714,31 +1765,94 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 				if (s == "addMethod") {
 					Node* nobj = marg;
 					Node* npred = g->Car()->Cdr();
-					return ObjSetMethoda(nobj, npred, module);
+					int r = ObjSetMethoda(nobj, npred, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <addMethod ");
+							npred->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "addMethoda") {
 					Node* nobj = marg;
 					Node* npred = g->Car()->Cdr();
-					return ObjSetMethoda(nobj, npred, module);
+					int r = ObjSetMethoda(nobj, npred, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <addMethoda ");
+							npred->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "addMethodz") {
 					Node* nobj = marg;
 					Node* npred = g->Car()->Cdr();
-					return ObjSetMethodz(nobj, npred, module);
+					int r = ObjSetMethodz(nobj, npred, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <addMethodz ");
+							npred->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "delMethod") {
 					Node* nobj = marg;
 					Node* npred = g->Car()->Cdr()->Car();
-					return ObjDelMethoda(nobj, npred, module);
+					int r = ObjDelMethoda(nobj, npred, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <delMethod ");
+							npred->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "delMethoda") {
 					Node* nobj = marg;
 					Node* npred = g->Car()->Cdr()->Car();
-					return ObjDelMethoda(nobj, npred, module);
+					int r = ObjDelMethoda(nobj, npred, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <delMethoda ");
+							npred->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "delMethodz") {
 					Node* nobj = marg;
 					Node* npred = g->Car()->Cdr()->Car();
-					return ObjDelMethodz(nobj, npred, module);
+					int r = ObjDelMethodz(nobj, npred, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <delMethodz ");
+							npred->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "delVar") {
 					Node* nobj = marg;
 					Node* nvar = g->Car()->Cdr()->Car();
-					return ObjDelVar(nobj, nvar, module);
+					int r = ObjDelVar(nobj, nvar, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <delVar ");
+							nvar->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				}
 			}
 		}
@@ -1752,7 +1866,16 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 				if (s == "cloneObj") {
 					Node* nobj = g->Car()->Cdr()->Car();
 					Node* nbase = marg;
-					return ObjCloneObj(nobj, nbase, module);
+					int r = ObjCloneObj(nobj, nbase, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <cloneObj ");
+							nbase->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				}
 			}
 		}
@@ -1803,6 +1926,13 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 		delete cx2;
 		cx2 = 0;
 
+		if (TraceFlag) {
+			mname->Car()->print();
+			printf(" : ");
+			gl->Car()->print();
+			printf("...success\n");
+		}
+	
 		return rn;
 	} else {
 		cxpop(cx2);
@@ -1817,6 +1947,169 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 	return 0;
 }
 
+static Node* nthrow_val = Nil;
+
+int Catch(Context* cx, Node* goalscar, List* module)
+{
+	int rn;
+
+	if (ListLength(goalscar->Cdr()) < 2) {
+		syserr("usage : <catch VAR PRED ...> \n");
+		return 0;
+	}
+
+	Node* nvar = goalscar->Cdr()->Car();
+	if (nvar->kind() != UNDEF) {
+		syserr("usage : <catch VAR PRED ...> \n");
+		return 0;
+	}
+	
+	Node* predarg = goalscar->Cdr()->Cdr();
+
+	int save_throwflag = ThrowFlag;
+	ThrowFlag = 0;
+
+	Context* cx2 = new Context(module, cx->modulename);
+	cx2->inherit = cx->inherit;
+	cx2->ioin = cx->ioin;
+	cx2->ioout = cx->ioout;
+	cx2->tokenflag = cx->tokenflag;
+
+	cxpush(cx2, goalscar);
+	cxpush(cx2, predarg);
+	cxpush(cx2, module);
+	if ((rn=Unify(cx2, predarg, module))>0) {
+		cx->Merge(cx2);
+
+		cxpop(cx2);
+		cxpop(cx2);
+		cxpop(cx2);
+		delete cx2;
+		cx2 = 0;
+
+		Node* env = Nil->Cons(Nil);
+
+		SetEnv(env, nvar);
+		((Undef*)(nvar->Val()))->Set(nthrow_val);
+
+		PushStack(cx, Nil, Nil, env);
+	
+		nthrow_val = Nil;
+		ThrowFlag = save_throwflag;
+		return rn;
+	} else {
+		cxpop(cx2);
+		cxpop(cx2);
+		cxpop(cx2);
+		delete cx2;
+		cx2 = 0;
+
+		nthrow_val = Nil;
+		ThrowFlag = save_throwflag;
+		return rn;
+	}
+}
+
+int Catch_SIGINT(Context* cx, Node* goalscar, List* module)
+{
+	int rn;
+
+	if (ListLength(goalscar->Cdr()) < 2) {
+		syserr("usage : <catch_sigint VAR PRED ...> \n");
+		return 0;
+	}
+
+	Node* nvar = goalscar->Cdr()->Car();
+	if (nvar->kind() != UNDEF) {
+		syserr("usage : <catch_sigint VAR PRED ...> \n");
+		return 0;
+	}
+	
+	Node* predarg = goalscar->Cdr()->Cdr();
+
+	extern void set_sigint_handler_catch_sigint();
+	extern void reset_sigint_handler();
+
+	set_sigint_handler_catch_sigint();
+	
+	int save_throwflag = ThrowFlag;
+	ThrowFlag = 0;
+
+	Context* cx2 = new Context(module, cx->modulename);
+	cx2->inherit = cx->inherit;
+	cx2->ioin = cx->ioin;
+	cx2->ioout = cx->ioout;
+	cx2->tokenflag = cx->tokenflag;
+
+	cxpush(cx2, goalscar);
+	cxpush(cx2, predarg);
+	cxpush(cx2, module);
+	if ((rn=Unify(cx2, predarg, module))>0) {
+		cx->Merge(cx2);
+
+		cxpop(cx2);
+		cxpop(cx2);
+		cxpop(cx2);
+		delete cx2;
+		cx2 = 0;
+
+		Node* env = Nil->Cons(Nil);
+
+		SetEnv(env, nvar);
+		((Undef*)(nvar->Val()))->Set(nthrow_val);
+
+		PushStack(cx, Nil, Nil, env);
+	
+		nthrow_val = Nil;
+		ThrowFlag = save_throwflag;
+
+		//reset_sigint_handler();
+		
+		return rn;
+	} else {
+		cxpop(cx2);
+		cxpop(cx2);
+		cxpop(cx2);
+		delete cx2;
+		cx2 = 0;
+
+		nthrow_val = Nil;
+		ThrowFlag = save_throwflag;
+
+		//reset_sigint_handler();
+		
+		return rn;
+	}
+}
+
+int EventThrow(Node* ev)
+{
+	nthrow_val = ev;
+
+	ThrowFlag = 1;
+
+	return 1;
+}
+
+int Throw(Context* cx, Node* goalscar, List* module)
+{
+	if (ListLength(goalscar->Cdr()) == 2) {
+		syserr("usage : <throw VAL> \n");
+		return 0;
+	}
+
+	int rn;
+
+	Node* ev = goalscar->Cdr()->Car()->Val();
+
+	if ((rn = FuncArg(cx, ev, goalscar, module)) <= 0) {
+		syserr("include: failed in the evaluation of the argument. \n");
+		return 0;
+	}
+
+	return EventThrow(ev);
+}
+
 int TimeOut(Context* cx, Node* goalscar, List* module)
 {
 #if 1
@@ -1864,7 +2157,7 @@ int TimeOut(Context* cx, Node* goalscar, List* module)
 		return -1;
 	}
 
-	Context* cx2 = new Context(module);
+	Context* cx2 = new Context(module, cx->modulename);
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
@@ -1913,7 +2206,7 @@ int DoLoop(Context* cx, Node* goalscar, List* module)
 		nresult = Nil;
 	}
 
-	Context *cx2 = new Context(module);
+	Context *cx2 = new Context(module, cx->modulename);
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
@@ -1948,6 +2241,9 @@ int DoLoop(Context* cx, Node* goalscar, List* module)
 			BreakFlag = 0;
 			break;
 		}
+		if (ThrowFlag) {
+			break;
+		}
 	} while (r>0);
 
 	cxpop(cx2);
@@ -1980,7 +2276,7 @@ int DoOrPred(Context* cx, Node* goalscar, List* module)
 		int r = 0;
 		
 		gl = goalscar->Car();
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
@@ -2013,7 +2309,7 @@ int DoAlt(Context* cx, Node* goalscar, List* module)
 	int	rn;
 	Node*	gl = MkList(goalscar->Cdr());
 
-	Context *cx2 = new Context(module);
+	Context *cx2 = new Context(module, cx->modulename);
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
@@ -2038,7 +2334,7 @@ int DoNot(Context* cx, Node* goalscar, List* module)
 	Node*	gl = goalscar->Cdr();
 	int	r;
 	
-	Context *cx2 = new Context(module);
+	Context *cx2 = new Context(module, cx->modulename);
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
@@ -2154,7 +2450,7 @@ int DoFor(Context* cx, Node* goalscar, List* module)
 	Node* env = Nil->Cons(Nil);
 	Context *cx2;
 	for (i = init_val; i <= to_val; i++) {
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
@@ -2212,6 +2508,9 @@ int DoFor(Context* cx, Node* goalscar, List* module)
 			BreakFlag = 0;
 			break;
 		}
+		if (ThrowFlag) {
+			break;
+		}
 	}		
 
 	if (nresult->kind() == UNDEF) {
@@ -2284,7 +2583,7 @@ int DoForeach(Context* cx, Node* goalscar, List* module)
 	Node* val = Nil;
 
 	for ( ; nlist->kind() != ATOM; nlist=nlist->Cdr()) {
-		Context *cx2 = new Context(module);
+		Context *cx2 = new Context(module, cx->modulename);
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
@@ -2346,6 +2645,9 @@ int DoForeach(Context* cx, Node* goalscar, List* module)
 			BreakFlag = 0;
 			break;
 		}
+		if (ThrowFlag) {
+			break;
+		}
 	}		
 
 	if (nresult->kind() == UNDEF) {
@@ -2452,7 +2754,7 @@ int DoFirstFor(Context* cx, Node* goalscar, List* module)
 	Node* env = Nil->Cons(Nil);
 	Context *cx2;
 	for (i = init_val; i <= to_val; i++) {
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
@@ -2580,7 +2882,7 @@ int DoFirstForeach(Context* cx, Node* goalscar, List* module)
 	Node* val = Nil;
 
 	for ( ; nlist->kind() != ATOM; nlist=nlist->Cdr()) {
-		Context *cx2 = new Context(module);
+		Context *cx2 = new Context(module, cx->modulename);
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
@@ -2667,7 +2969,7 @@ int DoFindAll(Context* cx, Node* goalscar, List* module)
 
 	gl = Append(gl, MkList(MkPred(mka("unknown"))));
 
-	Context *cx2 = new Context(module);
+	Context *cx2 = new Context(module, cx->modulename);
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
@@ -2867,7 +3169,7 @@ int ObjCloneObj(Node* nobj, Node* nbase, Node* module)
 		return 0;
 	}
 		
-	Asserta((List*)module, MkList(MkPred(Cons(MkPred(nobj), nval))));
+	Asserta((List*)module, Dup(MkList(MkPred(Cons(MkPred(nobj), nval)))));
 
 	return 1;
 }
@@ -3434,7 +3736,6 @@ int ObjSetVar(Node* nobj, Node* nvar, Node* val, List* module)
 				((List*)((Pred*)(md->Car()->Car()))->Node())
 					->SetCdr(Cons(MkList(
 					  MkPred(MkList(nvar, val))), n->Cdr()));
-
 				return 1;
 			}
 			Node* nprev = n; 
