@@ -11,29 +11,13 @@ using namespace std;
 #include "nwcc.h"
 #include "nwcclex.h"
 
-/*
- * parse program copyright (C) 2009 H.Niwa
- */
 
-/*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
- */
 
 #include "config.h"
 
+#include <strings.h>
 #include <sys/time.h>
 
 #ifndef __MINGW32__
@@ -42,6 +26,8 @@ using namespace std;
 
 #include <setjmp.h>
 
+#include <complex>
+
 #include "syserr.h"
 #include "pllex.h"
 
@@ -54,44 +40,65 @@ using namespace std;
 #include "unify.h"
 #include "func.h"
 #include "sysmodule.h"
+#include "builtin.h"
 
 jmp_buf program_jb;
 
-Node *r_head;
-Node *r_literal;
-Node *r_body;
-Node *r_clause;
-Node *r_defobj;
-Node *r_inherit;
-Node *r_pred;
-Node *r_proc;
-Node *r_node;
-Node *r_term_node;
-Node *r_list;
-Node *r_loop;
-Node *r_alt;
-Node *r_var;
-Node *r_obj;
-Node *r_class;
-Node *r_term;
-Node *r_name;
-Node *r_expression;
-Node *r_exp_plsmns;
-Node *r_exp_muldiv;
-Node *r_proc_letf;
-Node *r_proc_leti;
-Node *r_proc_rpnf;
-Node *r_proc_rpni;
-Node *r_proc_comparef;
-Node *r_proc_comparei;
-Node *r_comparing;
-Node *r_comp_and;
-Node *r_comp_not;
-Node *r_comp_gl;
-Node *r_proc_format;
+Node *r_head = Nil;
+Node *r_literal = Nil;
+Node *r_body = Nil;
+Node *r_clause = Nil;
+Node *r_defobj = Nil;
+Node *r_inherit = Nil;
+Node *r_pred = Nil;
+Node *r_proc = Nil;
+Node *r_node = Nil;
+Node *r_term_node = Nil;
+Node *r_list = Nil;
+Node *r_loop = Nil;
+Node *r_alt = Nil;
+Node *r_var = Nil;
+Node *r_obj = Nil;
+Node *r_class = Nil;
+Node *r_term = Nil;
+Node *r_name = Nil;
+Node *r_expression = Nil;
+Node *r_exp_plsmns = Nil;
+Node *r_exp_muldiv = Nil;
+Node *r_proc_letf = Nil;
+Node *r_proc_leti = Nil;
+Node *r_proc_letc = Nil;
+Node *r_proc_rpnf = Nil;
+Node *r_proc_rpni = Nil;
+Node *r_proc_comparef = Nil;
+Node *r_proc_comparei = Nil;
+Node *r_comparing = Nil;
+Node *r_comp_and = Nil;
+Node *r_comp_not = Nil;
+Node *r_comp_gl = Nil;
+Node *r_proc_format = Nil;
+
+extern int ErrorStopFlag;
+extern int BreakFlag;
+extern int ThrowFlag;
 
 Node *varlist = Nil;
 
+#ifdef __MINGW32__
+void
+bcopy (void *IN, void *OUT, size_t N)
+{
+  char *in = (char *) IN;
+  char *out = (char *) OUT;
+
+  for (; N > 0; N--)
+    {
+      *(out++) = *(in++);
+    }
+}
+
+#endif
+
 void
 RegisterVar (Var * v)
 {
@@ -99,7 +106,7 @@ RegisterVar (Var * v)
 }
 
 Var *
-SearchVar (char *s)
+SearchVar (const char *s)
 {
   Node *p;
   for (p = varlist; p != Nil; p = p->Cdr ())
@@ -113,9 +120,9 @@ SearchVar (char *s)
 }
 
 Var *
-GetVar (char *s)
+GetVar (const char *s)
 {
-  if (strcmp (s, "_") != 0)
+  if (strncmp (s, "_", 1) != 0)
     {
       Var *v = SearchVar (s);
       if (v != NULL)
@@ -123,7 +130,7 @@ GetVar (char *s)
 	  return v;
 	}
     }
-  Var *r = new Var (s);
+  Var *r = NewVar (s);
   RegisterVar (r);
   return r;
 }
@@ -134,11 +141,60 @@ ClearVar ()
   varlist = Nil;
 }
 
+
+int
+CheckSJIS2nd ()
+{
+  extern FILE *RdFp;
+  extern int CharSJISLen (unsigned char c);
+  int c;
+  long tp, i;
+  int flg = 0;
+
+  tp = FilePointSave ();
+  for (i = tp - 2; i >= 0; i--)
+    {
+      fseek (RdFp, i, 0);
+      c = fgetc (RdFp);
+      if (c < ' ')
+	{
+	  break;
+	}
+    }
+  i++;
+
+  flg = 0;
+  fseek (RdFp, i, 0);
+  for (;; i++)
+    {
+      c = fgetc (RdFp);
+      if (i > tp - 2)
+	{
+	  break;
+	}
+
+      if (flg)
+	{
+	  flg = 0;
+	}
+      else if (CharSJISLen (c) > 1)
+	{
+	  flg = 1;
+	}
+    }
+
+  FilePointResume (tp);
+
+  return flg;
+}
+
 int
 NOTSPACE ()
 {
   extern int Char;
   extern int pllexCkSpace (int c);
+  extern std::string code;
+  extern int CharSJISLen (unsigned char c);
 
   C ();
 
@@ -167,6 +223,13 @@ NOTSPACE ()
     case '{':
     case '}':
     case '|':
+      if (code == "SJIS")
+	{
+	  if (CheckSJIS2nd ())
+	    {
+	      return 0;
+	    }
+	}
       UnGetChar (Char);
       return -1;
     }
@@ -237,8 +300,8 @@ HEXNUM ()
 }
 
 
-char *InputSuffix = "pl";
-char *OutputSuffix = "c";
+const char *InputSuffix = "pl";
+const char *OutputSuffix = "c";
 int
 NwccMain ()
 {
@@ -253,17 +316,22 @@ int
 program ()
 {
 
-  extern int ErrorStopFlag;
+
   ErrorStopFlag = 0;
+  ErrGoal = Nil;
+  BreakFlag = 0;
+  ThrowFlag = 0;
 
   if (setjmp (program_jb) != 0)
     {
-      CXNode = NULL;
       ErrorStopFlag = 0;
-//                                      printf("# %d : error\n", LineNumber());
+
+      ErrGoal = Nil;
+
       return 0;
     }
 
+
   ErrorFlg = 0;
   MESG = NULL;
   do
@@ -328,7 +396,7 @@ goal ()
   if (!ErrorFlg)
     {
 
-      Node *l_goalnodes;
+      Node *l_goalnodes = Nil;
       int orflag = 0;
       Node *l_goal = Nil;
       int r;
@@ -447,11 +515,10 @@ goal ()
 	{
 	  PrintNode ("? ", l_goal);
 	}
-
       incflag = 0;
 
-      Context cxr (Module);
-      Context *cx = &cxr;
+      Context *cx = new Context (Module,
+				 new Atom ("WORLD"));
 
       l_goal = l_goal->Val ();
 
@@ -463,21 +530,34 @@ goal ()
       times (&tmsbuf);
       progtime.start_utime = tmsbuf.tms_utime;
       progtime.start_stime = tmsbuf.tms_stime;
+      progtime.start_cutime = tmsbuf.tms_cutime;
+      progtime.start_cstime = tmsbuf.tms_cstime;
 #endif
 
+      jmp_buf save_jb;
+      bcopy (program_jb, save_jb, sizeof save_jb);
+
       cxpush (cx, l_goal);
-//                                 r = FuncPred(cx, l_goal);
-      r = Unify (cx, l_goal);
+
+      if ((r = setjmp (program_jb)) == 0)
+	{
+	  r = Unify (cx, l_goal);
+
+	}
+      else
+	{
+	  r = r - 2;
+	}
 
       cxpop (cx);
 
-      if (!incflag)
+      bcopy (save_jb, program_jb, sizeof save_jb);
+
+      extern int resultflg;
+      if (!incflag && resultflg)
 	{
-//                                      printf("\n");
-//                                      PrintNode("result -- ", l_goal);
 	  printf ("result -- \n");
 	  PPmoduleBody (1, l_goal);
-//                                      l_goal->print(); printf("\n");
 	  if (r == 1)
 	    {
 	      printf ("-- true\n");
@@ -487,19 +567,20 @@ goal ()
 	      printf ("-- false\n");
 	    }
 	  else
-	    {			// -1
+	    {
 	      printf ("-- unknown\n");
 	    }
 	}
       ClearVar ();
 
-//                                 PPmodule(Module); printf("\n");
-/*
-				   if (cx != 0) {
-					   cx->Clear();
-					   cx = 0;
-				   }
-*/
+      BreakFlag = 0;
+      ThrowFlag = 0;
+
+
+      delete cx;
+
+      ErrGoal = Nil;
+
       GC ();
 
     }
@@ -509,6 +590,7 @@ goal ()
 int
 literal ()
 {
+  r_literal = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   int pred ();
@@ -527,43 +609,62 @@ literal ()
 int
 declare ()
 {
+
+  r_defobj = Nil;
+  r_clause = Nil;
+  r_list = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
-  int defobj ();
-  ErrorFlg = defobj ();
-  MESG = "defobj is missing ";
-  if (ErrorFlg)
-    {
-    }
+  {
+    int nloop = 0;
+    do
+      {
+	long chktell1, chktell2;
+	chktell1 = FilePointSave ();
+	int defobj ();
+	ErrorFlg = defobj ();
+	MESG = "defobj is missing ";
+	if (ErrorFlg)
+	  {
+	  }
+	if (!ErrorFlg)
+	  {
+	    Assert (Module, r_defobj);
+	  }
+	if (ErrorFlg)
+	  {
+	    int clause ();
+	    ErrorFlg = clause ();
+	    MESG = "clause is missing ";
+	    if (ErrorFlg)
+	      {
+	      }
+	    if (!ErrorFlg)
+	      {
+		Assert (Module, r_clause);
+	      }
+	  }
+	if (ErrorFlg)
+	  {
+	    int list ();
+	    ErrorFlg = list ();
+	    MESG = "list is missing ";
+	    if (ErrorFlg)
+	      {
+	      }
+	    if (!ErrorFlg)
+	      {
+		Module->SetCar (Append (Module->Car (), r_list->Car ()));
+	      }
+	  }
+	if (!ErrorFlg)
+	  nloop++;
+      }
+    while (0);
+  }
   if (!ErrorFlg)
     {
-      Assert (Module, r_defobj);
-    }
-  if (ErrorFlg)
-    {
-      int clause ();
-      ErrorFlg = clause ();
-      MESG = "clause is missing ";
-      if (ErrorFlg)
-	{
-	}
-      if (!ErrorFlg)
-	{
-	  Assert (Module, r_clause);
-	}
-    }
-  if (ErrorFlg)
-    {
-      int list ();
-      ErrorFlg = list ();
-      MESG = "list is missing ";
-      if (ErrorFlg)
-	{
-	}
-      if (!ErrorFlg)
-	{
-	  Module->SetCar (Append (Module->Car (), r_list->Car ()));
-	}
     }
   return ErrorFlg;
 }
@@ -571,8 +672,11 @@ declare ()
 int
 clause ()
 {
+
+  r_clause = Nil;
   Node *l_clause = Nil;
   ClearVar ();
+
   ErrorFlg = 0;
   MESG = NULL;
   int head ();
@@ -621,6 +725,7 @@ clause ()
 int
 head ()
 {
+  r_head = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   int pred ();
@@ -640,7 +745,7 @@ int
 body ()
 {
 
-  Node *r_bodynodes;
+  Node *r_bodynodes = Nil;
   int orflag = 0;
   r_body = Nil;
 
@@ -742,7 +847,8 @@ int
 defobj ()
 {
 
-  Node *l_defobj;
+  r_defobj = Nil;
+  Node *l_defobj = Nil;
 
   ErrorFlg = 0;
   MESG = NULL;
@@ -760,15 +866,15 @@ defobj ()
 	}
       if (ErrorFlg)
 	ErrorStop (MESG);
-      int node ();
-      ErrorFlg = node ();
-      MESG = "node is missing ";
+      int obj_name ();
+      ErrorFlg = obj_name ();
+      MESG = "obj_name is missing ";
       if (ErrorFlg)
 	{
 	}
       if (ErrorFlg)
 	ErrorStop (MESG);
-      l_defobj = MkList (r_node);
+      l_defobj = MkList (MkPred (r_class));
       do
 	{
 	  long chktell1, chktell2;
@@ -832,6 +938,7 @@ defobj ()
 int
 inherit ()
 {
+  r_inherit = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("inherit");
@@ -865,9 +972,10 @@ int
 pred ()
 {
 
-  Node *l_pred;
+  Node *l_pred = Nil;
   int dotflg = 0;
-  Node *lastnode;
+  Node *lastnode = Nil;
+  r_pred = Nil;
 
   ErrorFlg = 0;
   MESG = NULL;
@@ -997,6 +1105,19 @@ pred ()
 				l_pred = r_proc_leti;
 			      }
 			  }
+			if (ErrorFlg)
+			  {
+			    int proc_letc ();
+			    ErrorFlg = proc_letc ();
+			    MESG = "proc_letc is missing ";
+			    if (ErrorFlg)
+			      {
+			      }
+			    if (!ErrorFlg)
+			      {
+				l_pred = r_proc_letc;
+			      }
+			  }
 			if (ErrorFlg)
 			  {
 			    int proc_format ();
@@ -1026,7 +1147,21 @@ pred ()
 				    }
 				  if (!ErrorFlg)
 				    {
+
 				      l_pred = lastnode = MkList (r_node);
+
+				      ErrorFlg = COMP (";");
+				      MESG = "\";\" is missing ";
+				      if (ErrorFlg)
+					{
+					}
+				      if (!ErrorFlg)
+					{
+					}
+				      ErrorFlg = 0;
+				      MESG = NULL;
+				      if (ErrorFlg)
+					ErrorStop (MESG);
 				      do
 					{
 					  long chktell1, chktell2;
@@ -1039,10 +1174,24 @@ pred ()
 					    }
 					  if (!ErrorFlg)
 					    {
+
 					      l_pred = Append (l_pred,
 							       lastnode =
 							       MkList
 							       (r_node));
+
+					      ErrorFlg = COMP (";");
+					      MESG = "\";\" is missing ";
+					      if (ErrorFlg)
+						{
+						}
+					      if (!ErrorFlg)
+						{
+						}
+					      ErrorFlg = 0;
+					      MESG = NULL;
+					      if (ErrorFlg)
+						ErrorStop (MESG);
 					    }
 					  chktell2 = FilePointSave ();
 					  if ((chktell1 == chktell2)
@@ -1070,8 +1219,22 @@ pred ()
 					    }
 					  if (ErrorFlg)
 					    ErrorStop (MESG);
+
 					  ((List *) lastnode)->
 					    SetCdr (r_node);
+
+					  ErrorFlg = COMP (";");
+					  MESG = "\";\" is missing ";
+					  if (ErrorFlg)
+					    {
+					    }
+					  if (!ErrorFlg)
+					    {
+					    }
+					  ErrorFlg = 0;
+					  MESG = NULL;
+					  if (ErrorFlg)
+					    ErrorStop (MESG);
 					}
 				      ErrorFlg = 0;
 				      MESG = NULL;
@@ -1126,8 +1289,9 @@ int
 list ()
 {
 
-  Node *l_listnodes;
+  Node *l_listnodes = Nil;
   int orflag = 0;
+  r_list = Nil;
   Node *l_list = Nil;
 
   ErrorFlg = 0;
@@ -1300,7 +1464,10 @@ list ()
 int
 obj ()
 {
-  Node *l_obj;
+
+  r_obj = Nil;
+  Node *l_obj = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("::");
@@ -1335,6 +1502,7 @@ obj ()
 int
 node ()
 {
+  r_node = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   {
@@ -1486,6 +1654,18 @@ node ()
 				    r_node = mka (TOKEN);
 				  }
 			      }
+			    if (ErrorFlg)
+			      {
+				ErrorFlg = COMP ("i");
+				MESG = "\"i\" is missing ";
+				if (ErrorFlg)
+				  {
+				  }
+				if (!ErrorFlg)
+				  {
+				    r_node = mka ("i");
+				  }
+			      }
 			    if (!ErrorFlg)
 			      nloop++;
 			  }
@@ -1551,6 +1731,18 @@ node ()
 				    r_node = mka ((char *) s.c_str ());
 				  }
 			      }
+			    if (ErrorFlg)
+			      {
+				ErrorFlg = COMP ("i");
+				MESG = "\"i\" is missing ";
+				if (ErrorFlg)
+				  {
+				  }
+				if (!ErrorFlg)
+				  {
+				    r_node = mka ("-i");
+				  }
+			      }
 			    if (!ErrorFlg)
 			      nloop++;
 			  }
@@ -1568,6 +1760,18 @@ node ()
 	      {
 	      }
 	  }
+	if (ErrorFlg)
+	  {
+	    ErrorFlg = COMP ("i");
+	    MESG = "\"i\" is missing ";
+	    if (ErrorFlg)
+	      {
+	      }
+	    if (!ErrorFlg)
+	      {
+		r_node = mka ("i");
+	      }
+	  }
 	if (ErrorFlg)
 	  {
 	    int HEXNUM ();
@@ -1622,8 +1826,9 @@ int
 loop ()
 {
 
-  Node *l_loopnodes;
+  Node *l_loopnodes = Nil;
   int orflag = 0;
+  r_loop = Nil;
   Node *l_loop = Nil;
 
   ErrorFlg = 0;
@@ -1635,6 +1840,20 @@ loop ()
     }
   if (!ErrorFlg)
     {
+      int var ();
+      ErrorFlg = var ();
+      MESG = "var is missing ";
+      if (ErrorFlg)
+	{
+	}
+      if (!ErrorFlg)
+	{
+	  l_loop = MkList (r_var);
+	}
+      ErrorFlg = 0;
+      MESG = NULL;
+      if (ErrorFlg)
+	ErrorStop (MESG);
       do
 	{
 	  long chktell1, chktell2;
@@ -1731,7 +1950,7 @@ loop ()
 	}
       if (ErrorFlg)
 	ErrorStop (MESG);
-      r_loop = MkPred (Cons (mka ("loop"), MkList (l_loop)));
+      r_loop = MkPred (Cons (mka ("loop"), l_loop));
     }
   return ErrorFlg;
 }
@@ -1740,8 +1959,9 @@ int
 alt ()
 {
 
-  Node *l_altnodes;
+  Node *l_altnodes = Nil;
   int orflag = 0;
+  r_alt = Nil;
   Node *l_alt = Nil;
 
   ErrorFlg = 0;
@@ -1858,6 +2078,7 @@ alt ()
 int
 var ()
 {
+  r_var = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   {
@@ -1913,7 +2134,7 @@ var ()
 		      if (ErrorFlg)
 			ErrorStop (MESG);
 
-		      string buf = "#";
+		      std::string buf = "#";
 		      buf = buf + TOKEN;
 		      r_var = GetVar ((char *) buf.c_str ());
 
@@ -1941,17 +2162,45 @@ var ()
 int
 obj_name ()
 {
+  r_class = Nil;
   ErrorFlg = 0;
   MESG = NULL;
-  int Word ();
-  ErrorFlg = Word ();
-  MESG = "Word is missing ";
-  if (ErrorFlg)
-    {
-    }
+  {
+    int nloop = 0;
+    do
+      {
+	long chktell1, chktell2;
+	chktell1 = FilePointSave ();
+	int Word ();
+	ErrorFlg = Word ();
+	MESG = "Word is missing ";
+	if (ErrorFlg)
+	  {
+	  }
+	if (!ErrorFlg)
+	  {
+	    r_class = mka (TOKEN);
+	  }
+	if (ErrorFlg)
+	  {
+	    int var ();
+	    ErrorFlg = var ();
+	    MESG = "var is missing ";
+	    if (ErrorFlg)
+	      {
+	      }
+	    if (!ErrorFlg)
+	      {
+		r_class = r_var;
+	      }
+	  }
+	if (!ErrorFlg)
+	  nloop++;
+      }
+    while (0);
+  }
   if (!ErrorFlg)
     {
-      r_class = mka (TOKEN);
     }
   return ErrorFlg;
 }
@@ -1959,6 +2208,7 @@ obj_name ()
 int
 term ()
 {
+  r_term = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   int Word ();
@@ -1977,6 +2227,7 @@ term ()
 int
 name ()
 {
+  r_name = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   int Word ();
@@ -1995,7 +2246,10 @@ name ()
 int
 expression ()
 {
-  Node *l_expression;
+
+  r_expression = Nil;
+  Node *l_expression = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   int exp_plsmns ();
@@ -2073,7 +2327,10 @@ expression ()
 int
 exp_plsmns ()
 {
-  Node *l_exp_plsmns;
+
+  r_exp_plsmns = Nil;
+  Node *l_exp_plsmns = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   int exp_muldiv ();
@@ -2164,7 +2421,10 @@ exp_plsmns ()
 int
 term_node ()
 {
+
   std::string s;
+  r_term_node = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("+");
@@ -2262,6 +2522,21 @@ term_node ()
 		  }
 	      }
 	    if (ErrorFlg)
+	      {
+		ErrorFlg = COMP ("i");
+		MESG = "\"i\" is missing ";
+		if (ErrorFlg)
+		  {
+		  }
+		if (!ErrorFlg)
+		  {
+
+		    s += "i";
+		    r_term_node = MkList (mka ((char *) s.c_str ()));
+
+		  }
+	      }
+	    if (ErrorFlg)
 	      {
 		int HEXNUM ();
 		ErrorFlg = HEXNUM ();
@@ -2307,6 +2582,7 @@ term_node ()
 int
 exp_muldiv ()
 {
+  r_exp_muldiv = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   {
@@ -2380,7 +2656,10 @@ exp_muldiv ()
 int
 comparing ()
 {
+
+  r_comparing = Nil;
   Node *l_comp_or = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   int comp_and ();
@@ -2421,7 +2700,10 @@ comparing ()
 int
 comp_and ()
 {
+
+  r_comp_and = Nil;
   Node *l_comp_and = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   int comp_not ();
@@ -2462,7 +2744,10 @@ comp_and ()
 int
 comp_not ()
 {
+
+  r_comp_not = Nil;
   Node *l_comp_not = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("not");
@@ -2503,7 +2788,10 @@ comp_not ()
 int
 comp_gl ()
 {
-  Node *l_comp_gl;
+
+  r_comp_gl = Nil;
+  Node *l_comp_gl = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   {
@@ -2730,7 +3018,10 @@ comp_gl ()
 int
 proc_letf ()
 {
-  Node *l_proc_letf;
+
+  r_proc_letf = Nil;
+  Node *l_proc_letf = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("letf");
@@ -2753,6 +3044,11 @@ proc_letf ()
       if (ErrorFlg)
 	{
 	}
+      if (!ErrorFlg)
+	{
+	}
+      ErrorFlg = 0;
+      MESG = NULL;
       if (ErrorFlg)
 	ErrorStop (MESG);
       l_proc_letf = MkList (mka ("letf"), r_var, mka ("="));
@@ -2766,7 +3062,6 @@ proc_letf ()
       if (ErrorFlg)
 	ErrorStop (MESG);
 
-      // r_expression->print(); printf("\n");
       r_proc_letf = l_proc_letf;
       r_proc_letf = Append (r_proc_letf, r_expression);
 
@@ -2775,51 +3070,13 @@ proc_letf ()
   return ErrorFlg;
 }
 
-int
-proc_leti2 ()
-{
-  Node *l_proc_leti;
-  ErrorFlg = 0;
-  MESG = NULL;
-  int var ();
-  ErrorFlg = var ();
-  MESG = "var is missing ";
-  if (ErrorFlg)
-    {
-    }
-  if (!ErrorFlg)
-    {
-      ErrorFlg = COMP ("=");
-      MESG = "\"=\" is missing ";
-      if (ErrorFlg)
-	{
-	}
-      if (ErrorFlg)
-	ErrorStop (MESG);
-      l_proc_leti = MkList (mka ("let"), r_var, mka ("="));
-
-      int expression ();
-      ErrorFlg = expression ();
-      MESG = "expression is missing ";
-      if (ErrorFlg)
-	{
-	}
-      if (ErrorFlg)
-	ErrorStop (MESG);
-
-      // r_expression->print(); printf("\n");
-      r_proc_leti = l_proc_leti;
-      r_proc_leti = Append (r_proc_leti, r_expression);
-
-
-    }
-  return ErrorFlg;
-}
-
 int
 proc_leti ()
 {
-  Node *l_proc_leti;
+
+  r_proc_leti = Nil;
+  Node *l_proc_leti = Nil;
+
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("let");
@@ -2860,7 +3117,6 @@ proc_leti ()
       if (ErrorFlg)
 	ErrorStop (MESG);
 
-      // r_expression->print(); printf("\n");
       r_proc_leti = l_proc_leti;
       r_proc_leti = Append (r_proc_leti, r_expression);
 
@@ -2869,9 +3125,183 @@ proc_leti ()
   return ErrorFlg;
 }
 
+int
+proc_leti2 ()
+{
+
+  r_proc_leti = Nil;
+  Node *l_proc_leti = Nil;
+
+  ErrorFlg = 0;
+  MESG = NULL;
+  int var ();
+  ErrorFlg = var ();
+  MESG = "var is missing ";
+  if (ErrorFlg)
+    {
+    }
+  if (!ErrorFlg)
+    {
+      {
+	int nloop = 0;
+	do
+	  {
+	    long chktell1, chktell2;
+	    chktell1 = FilePointSave ();
+	    {
+	      int nloop = 0;
+	      do
+		{
+		  long chktell1, chktell2;
+		  chktell1 = FilePointSave ();
+		  ErrorFlg = COMP ("=");
+		  MESG = "\"=\" is missing ";
+		  if (ErrorFlg)
+		    {
+		    }
+		  if (!ErrorFlg)
+		    {
+		      l_proc_leti = MkList (mka ("let"), r_var, mka ("="));
+
+		      int expression ();
+		      ErrorFlg = expression ();
+		      MESG = "expression is missing ";
+		      if (ErrorFlg)
+			{
+			}
+		      if (ErrorFlg)
+			ErrorStop (MESG);
+
+		      r_proc_leti = l_proc_leti;
+		      r_proc_leti = Append (r_proc_leti, r_expression);
+
+		    }
+		  if (!ErrorFlg)
+		    nloop++;
+		}
+	      while (0);
+	    }
+	    if (!ErrorFlg)
+	      {
+	      }
+	    if (ErrorFlg)
+	      {
+		{
+		  int nloop = 0;
+		  do
+		    {
+		      long chktell1, chktell2;
+		      chktell1 = FilePointSave ();
+		      r_proc_leti = MkList (r_var);
+		      ErrorFlg = 0;
+		      MESG = NULL;
+
+		      if (!ErrorFlg)
+			{
+			  do
+			    {
+			      long chktell1, chktell2;
+			      chktell1 = FilePointSave ();
+			      int node ();
+			      ErrorFlg = node ();
+			      MESG = "node is missing ";
+			      if (ErrorFlg)
+				{
+				}
+			      if (!ErrorFlg)
+				{
+				  r_proc_leti = Append (r_proc_leti,
+							MkList (r_node));
+				}
+			      chktell2 = FilePointSave ();
+			      if ((chktell1 == chktell2) && !ErrorFlg)
+				ErrorStop ("endless loop occured");
+			    }
+			  while (!ErrorFlg);
+			  ErrorFlg = 0;
+			  MESG = NULL;
+			  if (ErrorFlg)
+			    ErrorStop (MESG);
+			}
+		      if (!ErrorFlg)
+			nloop++;
+		    }
+		  while (0);
+		}
+		if (!ErrorFlg)
+		  {
+		  }
+	      }
+	    if (!ErrorFlg)
+	      nloop++;
+	  }
+	while (0);
+      }
+      if (ErrorFlg)
+	ErrorStop (MESG);
+    }
+  return ErrorFlg;
+}
+
+int
+proc_letc ()
+{
+
+  r_proc_letc = Nil;
+  Node *l_proc_letc = Nil;
+
+  ErrorFlg = 0;
+  MESG = NULL;
+  ErrorFlg = COMP ("letc");
+  MESG = "\"letc\" is missing ";
+  if (ErrorFlg)
+    {
+    }
+  if (!ErrorFlg)
+    {
+      int var ();
+      ErrorFlg = var ();
+      MESG = "var is missing ";
+      if (ErrorFlg)
+	{
+	}
+      if (ErrorFlg)
+	ErrorStop (MESG);
+      ErrorFlg = COMP ("=");
+      MESG = "\"=\" is missing ";
+      if (ErrorFlg)
+	{
+	}
+      if (!ErrorFlg)
+	{
+	}
+      ErrorFlg = 0;
+      MESG = NULL;
+      if (ErrorFlg)
+	ErrorStop (MESG);
+      l_proc_letc = MkList (mka ("letc"), r_var, mka ("="));
+
+      int expression ();
+      ErrorFlg = expression ();
+      MESG = "expression is missing ";
+      if (ErrorFlg)
+	{
+	}
+      if (ErrorFlg)
+	ErrorStop (MESG);
+
+      r_proc_letc = l_proc_letc;
+      r_proc_letc = Append (r_proc_letc, r_expression);
+
+
+    }
+  return ErrorFlg;
+}
+
 int
 proc_comparef ()
 {
+  r_proc_comparef = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("comparef");
@@ -2899,6 +3329,7 @@ proc_comparef ()
 int
 proc_comparei ()
 {
+  r_proc_comparei = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("compare");
@@ -2926,6 +3357,7 @@ proc_comparei ()
 int
 proc_rpnf ()
 {
+  r_proc_rpnf = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("rpnf");
@@ -3075,6 +3507,7 @@ proc_rpnf ()
 int
 proc_rpni ()
 {
+  r_proc_rpni = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("rpn");
@@ -3224,7 +3657,9 @@ proc_rpni ()
 int
 Word ()
 {
-  string s;
+
+  std::string s;
+
   ErrorFlg = 0;
   MESG = NULL;
   {
@@ -3386,6 +3821,7 @@ WordBlock ()
 int
 proc_format ()
 {
+  r_proc_format = Nil;
   ErrorFlg = 0;
   MESG = NULL;
   ErrorFlg = COMP ("%");
@@ -3414,15 +3850,15 @@ proc_format ()
       if (ErrorFlg)
 	ErrorStop (MESG);
       l_proc_format = Append (l_proc_format, MkList (mka (TOKEN)));
-      int Word ();
-      ErrorFlg = Word ();
-      MESG = "Word is missing ";
+      int node ();
+      ErrorFlg = node ();
+      MESG = "node is missing ";
       if (ErrorFlg)
 	{
 	}
       if (ErrorFlg)
 	ErrorStop (MESG);
-      r_proc_format = Append (l_proc_format, MkList (mka (TOKEN)));
+      r_proc_format = Append (l_proc_format, MkList (r_node));
     }
   return ErrorFlg;
 }
