@@ -14,13 +14,13 @@
 	<append #x (a b c) (d e f)> <!>
 	::sys<writenl test1 append OK>;
 <test1 append>
-	::sys<writenl test1 append NG>;
+	::sys<writenl test1 append "*NG*">;
 
 <test2 append>
 	<append (a b c d e f ) (a b c) #Y> <!>
 	::sys<writenl test2 append OK>;
 <test2 append>
-	::sys<writenl test2 append NG>;
+	::sys<writenl test2 append "*NG*">;
 
 
 <reverse () ()>;
@@ -33,12 +33,12 @@
 	<reverse #rv (a b c d e f)> <!>
 	::sys<writenl test3 reverse OK>;
 <test3 reverse> 
-	::sys<writenl test3 reverse NG>;
+	::sys<writenl test3 reverse "*NG*">;
 <test4 reverse>
 	<reverse (a b c d e f) #rv> <!>
 	::sys<writenl test4 reverse OK>;
 <test4 reverse> 
-	::sys<writenl test4 reverse NG>;
+	::sys<writenl test4 reverse "*NG*">;
 
 
 // test comment
@@ -57,13 +57,13 @@
 	<rev #rv (a b c d e f)/* comment/* comment */ */> <!>
 	::sys<writenl test5 comment OK>;
 <test5 comment>
-	::sys<writenl test5 comment NG>;
+	::sys<writenl test5 comment "*NG*">;
 <test6 comment>
 <rev // comment
 (a b c d e f) #rv><!>
 	::sys<writenl test6 comment OK>;
 <test6 comment>
-	::sys<writenl test6 comment NG>;
+	::sys<writenl test6 comment "*NG*">;
 	
 
 // test args
@@ -76,7 +76,7 @@
 <test7 args> 
 	::sys<args #x>
 	::sys<writenl test7 args #x>
-	::sys<writenl test7 args NG>;
+	::sys<writenl test7 args "*NG*">;
 
 // test include
 //?::sys<DLIBPATH "test.d/:.">;
@@ -87,25 +87,25 @@
 	<unify list <append #x (a b) (c d)>> <!>
 	::sys<writenl test8 include OK>;
 <test8 include> 
-	::sys<writenl test8 include NG>;
+	::sys<writenl test8 include "*NG*">;
 
 <test9 include>
 	::list <append #x (a b) (c d)> <!>
 	::sys<writenl test9 include OK>;
 <test9 include> 
-	::sys<writenl test9 include NG>;
+	::sys<writenl test9 include "*NG*">;
 
 <test10 include>
 	::list <reverse (a bc d e f g h i j k l) #x> <!>
 	::sys<writenl test10 include OK>;
 <test10 include> 
-	::sys<writenl test10 include NG>;
+	::sys<writenl test10 include "*NG*">;
 
 <test11 include>
 	<unify list <reverse #x (a bc d e f g h i j k l)>> <!>
 	::sys<writenl test11 include OK>;
 <test11 include> 
-	::sys<writenl test11 include NG>;
+	::sys<writenl test11 include "*NG*">;
 
 
 
@@ -116,7 +116,7 @@
 	<module #x> <!>
  	::sys<writenl test12 module OK>;
 <test12 module>
- 	::sys<writenl test12 module NG>;
+ 	::sys<writenl test12 module "*NG*">;
 
 // test obj
 
@@ -124,12 +124,12 @@
 	::penguin <swim> <!>
  	::sys<writenl test13 obj OK>;
 <test13 obj>
- 	::sys<writenl test13 obj NG>;
+ 	::sys<writenl test13 obj "*NG*">;
 <test14 obj>
 	<not ::penguin <fly>> <!>
  	::sys<writenl test14 obj OK>;
 <test14 obj>
- 	::sys<writenl test14 obj NG>;
+ 	::sys<writenl test14 obj "*NG*">;
 
 ::<bird
 	<fly>;
@@ -154,7 +154,7 @@
  	::sys<writenl test15 token OK>
 	;
 <test15 token>
- 	::sys<writenl test15 token NG>;
+ 	::sys<writenl test15 token "*NG*">;
 
 <test16 char>
 	::sys <char #x "abcdefghijklmn">
@@ -163,7 +163,7 @@
  	::sys<writenl test16 char OK>
 	;
 <test16 char>
- 	::sys<writenl test16 char NG>
+ 	::sys<writenl test16 char "*NG*">
 	;
 
 <test17 compare>
@@ -173,7 +173,7 @@
 	::sys<writenl test17 compare OK>
 	;
 <test17 compare>
- 	::sys<writenl test17 compare NG>
+ 	::sys<writenl test17 compare "*NG*">
 	;
 <test18 comparef>
 	<#y=1>
@@ -182,7 +182,7 @@
 	::sys<writenl test18 comparef OK>
 	;
 <test18 comparef>
- 	::sys<writenl test18 comparef NG>
+ 	::sys<writenl test18 comparef "*NG*">
 	;
 <test19 compare>
 	<#y=1>
@@ -191,7 +191,7 @@
 	::sys<writenl test19 compare OK>
 	;
 <test19 compare>
- 	::sys<writenl test19 compare NG>
+ 	::sys<writenl test19 compare "*NG*">
 	;
 <test20 comparef>
 	<#y=1>
@@ -200,7 +200,7 @@
 	::sys<writenl test20 comparef OK>
 	;
 <test20 comparef>
- 	::sys<writenl test20 comparef NG>
+ 	::sys<writenl test20 comparef "*NG*">
 	;
 <test21 compare>
 	<#y=1>
@@ -209,7 +209,7 @@
 	::sys<writenl test21 compare OK>
 	;
 <test21 compare>
- 	::sys<writenl test21 compare NG>
+ 	::sys<writenl test21 compare "*NG*">
 	;
 <test22 comparef>
 	<#y=1>
@@ -218,7 +218,7 @@
 	::sys<writenl test22 comparef OK>
 	;
 <test22 comparef>
- 	::sys<writenl test22 comparef NG>
+ 	::sys<writenl test22 comparef "*NG*">
 	;
 
 ?<test1 append>
