@@ -1,5 +1,5 @@
 /*
- * token operation program copyright (C) 2009 - 2010 H.Niwa
+ * token operation program copyright (C) 2009 - 2011 H.Niwa
  */
 
 /*
@@ -85,6 +85,7 @@ int tokenCountIndent(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 int tokenNextIndent(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 
 std::string	Tokenbuf = "";
+std::string	RToken   = "";
 
 int TokenVar(Context* cx, Node* goalscar)
 {
@@ -185,7 +186,7 @@ void tokenSkipSpace(Context* cx)
 	
 	for( ; c != EOF; ) {
 		c = fgetc(cx->ioin);
-		if ((c == '\n') || (c == '\t')) {
+		if ((c == '\n') || (c == '\t')||(c == 0x0d)||(c == 0x0a)) {
 			continue;
 		}
 		if ((c < ' ') || (c == EOF)) {
@@ -219,7 +220,7 @@ int token(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals, List* modu
 
 	tokenSkipSpace(cx);
 	
-	Context *cx2 = new Context(module);
+	Context *cx2 = new Context(module, cx->modulename);
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
@@ -301,6 +302,7 @@ int tokenCmpDirect(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals, N
 //printf("tokencmpdirect if %x %x \n", c, 0xff & s[i]);
 			fseek(cx->ioin, tellsave, SEEK_SET);
 
+			RToken = Tokenbuf;
 			Tokenbuf = "";
 //printf("tokencmpdirect return -1 %x %x \n", c, 0xff & s[i]);
 			return -1;
@@ -341,7 +343,9 @@ int tokenC(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		ungetc(c, cx->ioin);
 		return -1;
 	}
-	Tokenbuf = (char)c;
+
+	char cbuf[2] = {c, 0};
+	Tokenbuf = cbuf;
 
 	n = CharLen(c);
 	if (n > 1) {
@@ -384,16 +388,21 @@ int tokenEof(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return 0;
 	}
 
+	long tellsave = ftell(cx->ioin);
+	
+	tokenSkipSpace(cx);
+
 	int c = fgetc(cx->ioin);
 	if (c != EOF) {
-		ungetc(c, cx->ioin);
+		// ungetc(c, cx->ioin);
+		fseek(cx->ioin, tellsave, SEEK_SET);
 		return -1;
 	}
 
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};
+	Tokenbuf = cbuf;
 
 	return 1;
-
 }
 
 int tokenN(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
@@ -419,7 +428,8 @@ int tokenN(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	if (ll == 1) {
 		return 1;
@@ -462,12 +472,13 @@ int tokenA(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 		
+	char cbuf[2] = {c, 0};
+	Tokenbuf = cbuf;
+
 	if (ll == 1) {
 		return 1;
 	}
 
-	Tokenbuf = (char)c;
-
 	if (goalscar->Cdr()->Car()->kind() != UNDEF) {
 		syserr("usage : <A [VAR]> \n");
 		return 0;
@@ -506,7 +517,8 @@ int tokenAN(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 		
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	if (ll == 1) {
 		return 1;
@@ -576,7 +588,8 @@ int tokenLineEnd(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	return 1;
 }
@@ -616,7 +629,8 @@ int tokenCR(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	return 1;
 }
@@ -640,7 +654,8 @@ int tokenCNTL(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	if (ll == 1) {
 		return 1;
@@ -654,7 +669,6 @@ int tokenCNTL(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	Node* env=Nil->Cons(Nil);
 	PushStack(cx, goalscar, Nil, env);
 
-	Tokenbuf = (char)c;
 	return TokenVar(cx, goalscar);
 }
 
@@ -723,7 +737,8 @@ int tokenPunct(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return -1;
 	}
 
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	if (ll == 1) {
 		return 1;
@@ -737,7 +752,6 @@ int tokenPunct(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	Node* env=Nil->Cons(Nil);
 	PushStack(cx, goalscar, Nil, env);
 
-	Tokenbuf = (char)c;
 	return TokenVar(cx, goalscar);
 
 }
@@ -969,7 +983,7 @@ int tokenWildcard(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		std::string Savetoken = Tokenbuf;
 		if (cx->tokenflag) cx->token = cx->token + Tokenbuf;
 		
-		Context *cx2 = new Context(cx->module);
+		Context *cx2 = new Context(cx->module, cx->modulename);
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
@@ -1163,7 +1177,8 @@ int tokenFNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	if (c == '.') {
 		dot++;
 	}
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 
 	for (;;) {
 		c = fgetc(cx->ioin);
@@ -1535,7 +1550,8 @@ int tokenRangeSub(Context* cx, Node* goalscar, int flg, List* module)
 
 	c = fgetc(cx->ioin);
 	n = CharLen(c);
-	Tokenbuf = (char)c;
+	char cbuf[2] = {c, 0};        
+	Tokenbuf = cbuf;        
 	cl = 0xff & c;
 	for (i = 1; i < n; i++) {
 		cl <<= 8;
@@ -1812,7 +1828,6 @@ int tokenNextStr(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	}
 
 	int	n;
-	std::string s;
 		
 	int ll = ListLength(goalscar);
 	if (ll != 2) {
@@ -1863,7 +1878,6 @@ int tokenNotNextStr(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	}
 
 	int	n;
-	std::string s;
 		
 	int ll = ListLength(goalscar);
 	if (ll != 2) {
@@ -1879,9 +1893,6 @@ int tokenNotNextStr(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	std::string str;
 	((Atom*)(goalscar->Cdr()->Car()->Val()))->toString(str);
 
-	std::string savecxtoken = cx->token;
-	std::string saveTokenbuf = Tokenbuf;
-	
 	long fpo = ftell(cx->ioin);
 
 	tokenSkipSpace(cx);
@@ -1900,9 +1911,6 @@ int tokenNotNextStr(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 			
 	fseek(cx->ioin, fpo, 0);
 	
-	cx->token = savecxtoken;
-	Tokenbuf = saveTokenbuf;
-		
 	return f;
 }
 
@@ -1932,7 +1940,7 @@ int tokenNextSyntax(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 
 	int rn;
 
-	Context *cx2 = new Context(cx->module);
+	Context *cx2 = new Context(cx->module, cx->modulename);
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
@@ -1983,7 +1991,7 @@ int tokenNotNextSyntax(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goal
 
 	int rn;
 
-	Context *cx2 = new Context(cx->module);
+	Context *cx2 = new Context(cx->module, cx->modulename);
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
